"""
Tento program:
- Načte data o publikacích ze vstupu (rok, časopis, název, autoři)
- Vytřídí pouze publikace od roku 2015 a novější
- Vypíše počet publikací ke každému autorovi (abecedně)
- TODO: Vypíše autora s největším počtem publikací

Spuštění programu:
- Z příkazového řádku:
cat data.txt | python papers.py
(Tento zápis znamená, že nemusíme vstup psát ručně, ale načte se ze souboru data.txt)

Úkoly:
1. Zhezčit zápis programu (stylistická doporučení (např. mezery před a za =, pojmenování proměnných)
2. Zpřehlednit program rozdělením do funkcí
3. Zefektivnit program tak, aby běžel rychle i na velkém datovém souboru (data_big.txt)
4. Zjednodušit program pomocí rozbalení, generátorových výrazů, případně funkcí enumerate, sorted...
5. Doplnit poslední krok (výběr autora s největším počtem publikací)

Rozšiřující učivo:
6. Zpřehlednit program použitím NamedTuple
7. Upravit program použitím walrus operator (:=), generátorové funkce, lambda funkce (zde nutně nezefektivní ani nezpřehlední, spíš věc vkusu)
"""

from __future__ import annotations


def log(name: str, items: list | dict) -> None:
    """Print length of a list/dict and its first item"""
    print(f'{name}: {len(items)}')
    if len(items) > 0:
        if isinstance(items, dict):
            for key, value in items.items():
                print(f'{key}: {value}')
                break
        else:
            print(items[0])
    print()


def read_lines() -> list[str]:
    """Read lines from input until the first empty line"""
    lines = []
    line = input()
    while line != '':
        lines.append(line)
        line = input()
    return lines


def parse_paper(line: str) -> tuple[int, str, str, list[str]]:
    """Parse line as a paper in this format: year;journal;title;authors..."""
    fields = line.split(';')
    year = fields[0]
    journal = fields[1]
    title = fields[2]
    authors = []
    for i in range(3, len(fields)):
        authors.append(fields[i])
    return (int(year), journal, title, authors)


def parse_papers(lines: list[str]) -> list[tuple[int, str, str, list[str]]]:
    """Parse lines as papers in this format: year;journal;title;authors..."""
    papers = []
    for line in lines:
        papers.append(parse_paper(line))
    return papers


def filter_newest_papers(papers: list[tuple[int, str, str, list[str]]], min_year: int) -> list[tuple[int, str, str, list[str]]]:
    """Return the list of papers published in year `min_year` and newer."""
    filtered = []
    for paper in papers:
        if paper[0] >= min_year:
            filtered.append(paper)
    return filtered


def get_authors(papers: list[tuple[int, str, str, list[str]]]) -> list[str]:
    """Return the list of all authors of papers (in alphabetical order)."""
    author_set = set()
    for paper in papers:
        for author in paper[-1]:
            author_set.add(author)
    authors = list(author_set)
    authors.sort()
    return authors


def sort_paper_by_author(papers: list[tuple[int, str, str, list[str]]]) -> dict[str, list[tuple[int, str, str, list[str]]]]:
    """Return the dict mapping authors to list of their papers."""
    authors = get_authors(papers)
    papers_per_author = {}
    for author in authors:
        papers_per_author[author] = []
    for paper in papers:
        for author in paper[-1]:
            papers_per_author[author].append(paper)
    return papers_per_author


def print_overview(papers_per_author: dict[str, list[tuple[int, str, str, list[str]]]]) -> None:
    """For each author, print their name and number of papers."""
    print('Overview of number of papers per author:')
    for author, papers in papers_per_author.items():
        print(f'    {author}: {len(papers)}')
    print()


def get_top_author(papers_per_author: dict[str, list[tuple[int, str, str, list[str]]]]) -> str:
    """Get the author with the highest number published papers."""
    ...  # TODO


def main():
    lines = read_lines()
    log('Read lines', lines)

    all_papers = parse_papers(lines)
    log('Parsed papers', all_papers)

    filtered_papers = filter_newest_papers(all_papers, 2015)
    log('Filtered papers', filtered_papers)

    all_authors = get_authors(filtered_papers)
    log('Authors', all_authors) 

    papers_per_author = sort_paper_by_author(filtered_papers)
    log('Assigned papers to authors', papers_per_author)

    print_overview(papers_per_author)

    print('The author with most papers:')
    print(f'    {get_top_author(papers_per_author)}')
    print()


main()

