"""
Tento program:
- Načte data o publikacích ze vstupu (rok, časopis, název, autoři)
- Vytřídí pouze publikace od roku 2015 a novější
- Vypíše počet publikací ke každému autorovi (abecedně)
- TODO: Vypíše autora s největším počtem publikací

Spuštění programu:
- Z příkazového řádku:
cat data.txt | python papers.py
(Tento zápis znamená, že nemusíme vstup psát ručně, ale načte se ze souboru data.txt)

Úkoly:
1. Zhezčit zápis programu (stylistická doporučení (např. mezery před a za =, pojmenování proměnných)
2. Zpřehlednit program rozdělením do funkcí
3. Zefektivnit program tak, aby běžel rychle i na velkém datovém souboru (data_big.txt)
4. Zjednodušit program pomocí rozbalení, generátorových výrazů, případně funkcí enumerate, sorted...
5. Doplnit poslední krok (výběr autora s největším počtem publikací)

Rozšiřující učivo:
6. Zpřehlednit program použitím NamedTuple
7. Upravit program použitím walrus operator (:=), generátorové funkce, lambda funkce (zde nutně nezefektivní ani nezpřehlední, spíš věc vkusu)
"""

from __future__ import annotations


def log(name: str, items: list | dict) -> None:
    """Print length of a list/dict and its first item"""
    print(f'{name}: {len(items)}')
    if len(items) > 0:
        if isinstance(items, dict):
            for key, value in items.items():
                print(f'{key}: {value}')
                break
        else:
            print(items[0])
    print()


# Read lines
lines=[ ]
x=input( )
while x != '':
    lines = lines + [x]
    x=input ()

log('Read lines', lines)


# Parse lines
all_papers=[ ]
for x in lines:
    fields=x.split (';')
    a=fields[0]  # year
    b=fields[1]  # jounal
    c=fields[2]  # title of the paper
    d=[]  # list of authors
    for i in range(3, len(fields)):
        d.append(fields[i])
    all_papers.append([int(a),b,c,d])

log('Parsed papers', all_papers)


# Filter papers by year
filtered_papers = []
for paper in all_papers:
    if paper[0] >= 2015:
        filtered_papers.append(paper)

log('Filtered papers', filtered_papers)


# Get all authors
all_authors=[]
for paper in filtered_papers:
    for author in paper[-1]:
        if author not in all_authors:
            all_authors.append(author)
all_authors.sort()

log('Authors', all_authors)


# Assign papers to authors
papers_per_author = {}
for author in all_authors:
    authors_papers = []
    for paper in filtered_papers:
        if author in paper[-1]:
            authors_papers.append(paper)
    papers_per_author.update({author: authors_papers})

log('Assigned papers to authors', papers_per_author)


# Print overview
print('Overview of number of papers per author:')
for author in all_authors:
    print('    '+author+': '+str(len(papers_per_author[author])))
print()


# Print author with the most papers
top_author = ... # TODO

print('The author with most papers:')
print(f'    {top_author}')
print()


# Řešení (postupné úpravy programu) najdete ve složce `papers-reseni`. 
# Rozdíly mezi dvěma verzemi lze zobrazit ve VSCode 
# (kliknout pravým na první soubor a zvolit Select for Compare, 
# pak kliknout pravým na druhý soubor a zvolit Compare with Selected).